# This script produces figures 
# to be used in the manuscript

#### Set up ####
rm(list = ls())
library("survival")
library("ggplot2")
library(ggpubr)
library("cubature")
library(tidyverse)
library(dplyr)
library(here)
library(risksetROC)
library(mgcv)
library(scam)
library("clinfun")
library(glmnet)
library(kableExtra)
theme_set(theme_minimal())
cbPalette <- c("#999999", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7")

# set the seed
set.seed(512)

#### Overfit ####
# load simulation results
load(here("Data/SimNoiseModel.RData"))

tv_auc_df <- bind_rows(auc_lst, .id = "iter") 
c_df <- bind_rows(c_lst, .id="iter") 

# load true values 
load(here("Data/true_values.RData"))
# re-interpolate true AUC
true_auc_df <- approx(x = true_auc_sort$time_bin, y = true_auc_sort$auc, 
                      xout = unique(tv_auc_df$time))
true_auc_df <- data.frame(true_auc_df) %>% 
  rename("time" = x, "true_auc" = y)

# figure of I/D AUC
tv_auc_df %>% 
  # filter(model != "20 noise lasso" & model != "100 noise lasso") %>%
  mutate(model = factor(model, 
                        levels = c("No noise", "20 noise", "100 noise"))) %>%
  left_join(true_auc_df, by = "time") %>% 
  pivot_longer(3:5, names_to = "estimator", values_to = "AUC") %>% 
  mutate(estimator = factor(estimator, levels = c("HZ", "NP", "SNP"),
                            labels = c("Semi-parametric", 
                                       "Non-parametric",
                                       "Smoothed non-parametric"))) %>%
  ggplot()+
  geom_smooth(aes(x=time, y=AUC, col=model, linetype=sample), 
              formula = y~s(x, k=30, bs = "cs"), method = "gam", se=F, na.rm = T)+
  geom_line(aes(x=time, y=true_auc), col = "black", size = 0.5)+
  scale_color_manual(values = cbPalette)+
  labs(col = "Model", linetype = "Sample", x = "Time")+
  facet_wrap(~estimator, nrow = 1)+
  theme(axis.text.x = element_text(vjust = 0.9))
ggsave(filename = here("Figs/tv_auc_noise.pdf"), 
       width=12, height=4, bg="white", dpi = 300)

# figure of variation of AUC
tv_auc_df %>% 
  # filter(model != "20 noise lasso" & model != "100 noise lasso") %>%
  mutate(model = factor(model, 
                        levels = c("No noise", "20 noise", "100 noise"))) %>%
  pivot_longer(c("HZ", "NP", "SNP"), names_to = "Estimator", values_to = "AUC") %>%
  mutate(Estimator = factor(Estimator, levels = c("HZ", "NP", "SNP"),
                            labels = c("Semi-parametric", 
                                       "Non-parametric",
                                       "Smoothed non-parametric"))) %>%
  mutate(time_bin = cut(time, breaks = seq(0, 1, 0.2), include.lowest = T)) %>%
  ggplot(aes(x = factor(time_bin), y = AUC, fill = Estimator))+
  geom_boxplot(outlier.size = 0.5)+
  facet_grid(rows = vars(sample), cols = vars(model))+
  labs(x = "Time", y= "AUC")+
  ylim(0, 1)+
  theme(axis.text.x = element_text(vjust = 0.9, angle = 45),
        text = element_text(size = 12.5), 
        legend.position = "top")+
  scale_fill_manual(values = cbPalette)+
  scale_color_manual(values = cbPalette)
ggsave(filename = here("Figs/tv_auc_noise_var.pdf"), 
       width=12, height=8, bg="white", dpi = 300)


# a look at the variation at each time point
tv_auc_df %>% 
  # filter(model != "20 noise lasso" & model != "100 noise lasso") %>%
  mutate(model = factor(model, 
                        levels = c("No noise", "20 noise", "100 noise"))) %>%
  pivot_longer(c("HZ", "NP", "SNP"), names_to = "Estimator", values_to = "AUC") %>%
  mutate(Estimator = factor(Estimator, levels = c("HZ", "NP", "SNP"),
                            labels = c("Semi-parametric", 
                                       "Non-parametric",
                                       "Smoothed non-parametric"))) %>%
  mutate(time_bin = cut(time, breaks = seq(0, 1, 0.2), include.lowest = T)) %>% 
  group_by(time_bin, Estimator) %>% 
  filter(model=="No noise") %>%
  summarise_at("AUC", sd, na.rm=T)


# figure of concordance
c_df %>% 
  # filter(model != "20 noise lasso" & model != "100 noise lasso") %>%
  pivot_longer(2:6, names_to = "estimator", values_to = "Concordance") %>%
  mutate(sample = factor(sample, levels =c("In-sample", "Out-of-sample"))) %>%
  mutate(model = factor(model, 
                        levels = c("No noise", "20 noise", "100 noise")),
         estimator = factor(estimator, 
                            levels = c("HZ", "GH", "Harrell.s", "NP", "SNP"), 
                            labels = c("HZ", "GH", "Harrell", "NP", "SNP"), )) %>%
  mutate(type = ifelse(estimator=="HZ"|estimator=="GH", "Semi-parametric", "Non-parametric")) %>%
  mutate(type = factor(type, levels = c("Semi-parametric", "Non-parametric"))) %>%
  rename(Sample = sample) %>%
  ggplot()+
  geom_boxplot(aes(x=model, y=Concordance, fill=Sample), na.rm = T)+
  geom_hline(yintercept = true_c, col = "black")+
  facet_wrap(~type+estimator, nrow = 1)+
  labs(x = "Model")+
  theme(axis.text.x = element_text(angle=45, size = 12.5),
        text = element_text(size = 12.5))+
  scale_color_manual(values = cbPalette)+
  scale_fill_manual(values = cbPalette)
ggsave(filename = here("Figs/c_noise.pdf"), 
       width=12, height=4, bg="white", dpi = 300)


#### Covariate misaglinment ####
# load simulation results
load(here("Data/SimContamData.RData"))
tv_auc_df2 <- bind_rows(auc_lst2) 
c_df2 <- bind_rows(c_lst2)

# load true values
load(here("Data/true_values.RData"))
# use interpolation to estimate AUC on simulated time series
true_auc_df <- approx(x = true_auc_sort$time_bin, y = true_auc_sort$auc, 
                      xout = unique(tv_auc_df2$time))
true_auc_df <- data.frame(true_auc_df) %>% 
  rename("time" = x, "true_auc" = y)

# figure of AUC
tv_auc_df2 %>% 
  left_join(true_auc_df, by = "time") %>%
  mutate(outlier = ifelse(is.na(outlier), "None", outlier)) %>%
  pivot_longer(2:4, names_to = "Estimator", values_to = "AUC") %>%
  mutate(Estimator = factor(Estimator, levels = c("HZ", "NP", "SNP"),
                            labels = c("Semi-parametric", 
                                       "Non-parametric",
                                       "Smoothed non-parametric"))) %>%
  ggplot()+
  geom_smooth(aes(x=time, y=AUC, col=outlier, linetype=sample), 
              formula = y~s(x, k=30, bs = "cs"), method = "gam", se=F, na.rm = T)+
  geom_line(aes(x=time, y=true_auc), col = "black", size = 0.5)+
  facet_wrap(~Estimator, nrow = 1)+
  scale_color_manual(values = cbPalette)+
  labs(col = "Misalignment", linetype = "Sample", x = "Time")+
  theme(axis.text.x = element_text(vjust = 0.9))
ggsave(filename = here("Figs/tv_auc_contaim.pdf"), 
       width=12, height=4, bg="white", dpi = 300)

# figure of variaton of AUC
tv_auc_df2 %>%
  mutate(time_bin = cut(time, breaks = seq(0, 1, 0.2), include.lowest = T)) %>%
  mutate(outlier = ifelse(is.na(outlier), "", outlier)) %>%
  mutate(outlier = relevel(as.factor(outlier), ref = "None")) %>%
  pivot_longer(c("HZ", "NP", "SNP"), names_to = "Estimator", values_to = "AUC") %>%
  mutate(Estimator = factor(Estimator, levels = c("HZ", "NP", "SNP"),
                            labels = c("Semi-parametric", 
                                       "Non-parametric",
                                       "Smoothed non-parametric"))) %>%
  ggplot()+
  geom_boxplot(aes(x=time_bin, y=AUC, fill = Estimator),
               outlier.size = 0.2, show.legend = F)+
  ylim(0, 1)+
  facet_wrap(~sample+outlier, nrow = 1)+
  theme(axis.text.x = element_text(vjust = 0.9, angle = 45),
        text = element_text(size = 12.5))+
  scale_fill_manual(values = cbPalette)+
  scale_color_manual(values = cbPalette)+
  labs(x="Time", y="AUC")
ggsave(filename = here("Figs/tv_auc_contam_var.pdf"), 
       width=12, height=4, bg="white", dpi = 300)

# figure of concordance
c_df2 %>% 
  pivot_longer(1:5, names_to = "estimator", values_to = "Concordance") %>%
  mutate(sample = factor(sample, levels =c("In-sample", "Out-of-sample"))) %>%
  mutate(outlier = ifelse(is.na(outlier), "In-sample", outlier)) %>%
  mutate(outlier = factor(outlier, 
                          levels = c("In-sample", "None", "N(0, 5)", "N(5, 1)")),
         estimator = factor(estimator, 
                            levels = c("HZ", "GH", "Harrell.s", "NP", "SNP"), 
                            labels = c("HZ", "GH", "Harrell", "NP", "SNP"), )) %>%
  mutate(type = ifelse(estimator=="HZ"|estimator=="GH", "Semi-parametric", "Non-parametric")) %>%
  mutate(type = factor(type, levels = c("Semi-parametric", "Non-parametric"))) %>%
  rename(Sample = sample) %>%
  ggplot()+
  geom_boxplot(aes(x=outlier, y=Concordance, fill=Sample), na.rm = T)+
  geom_hline(yintercept = true_c, col = "black")+
  facet_wrap(~type+estimator, nrow = 1)+
  labs(x="", y="Concordance")+
  theme(axis.text.x = element_text(angle=45, vjust = 0.9, size = 12.5),
        text = element_text(size = 12.5))+
  scale_color_manual(values = cbPalette)+
  scale_fill_manual(values = cbPalette)
ggsave(filename = here("Figs/c_contam.pdf"), 
       width=12, height=4, bg="white", dpi = 300)
